// Created by Josh Heyde
// Last updated 27/01/09

#include "C_Route.h"
#include "C_Vector.h"
#include <fstream>
#include <iostream>
#include <assert.h>

// constructor
C_Route::C_Route(string file)
: fileName(file), numOfWayPoints(0)
{
}

// destructor
C_Route::~C_Route()
{
	// deletes all created waypoints
	for (std::vector<C_Vector*>::const_iterator iter = v_wayPointVector.begin(); iter < v_wayPointVector.end(); iter++)
		delete *iter;
}

// other member functions

// intialising the routes from a file
bool C_Route::initialise()
{
	// error handling statements
	// asserting certain statements for this function
	assert(fileName.empty() == false);

	// creating an object to read in the route data from
	ifstream wayPoints(fileName.c_str(), ios::in);

	// testing to see if the file opened correctly
	if(wayPoints.fail())
		return false;

	while(!wayPoints.eof())
	{
		// creates a temporary vector pointer to pass onto the vector
		C_Vector* temp = new C_Vector(0, 0);

		// getting the current wayPoint from a file
		wayPoints >> temp->x >> temp->y;

		// pushes the waypoint onto the vector
		v_wayPointVector.push_back(temp);

		// increases the number of waypoints for this route
		numOfWayPoints++;
	}

	// closes the waypoints file object
	wayPoints.close();

	// shows the route has been successfully created
	return true;
}

// mutator functions
void C_Route::setFileName(const std::string &newFileName)
{
	fileName = newFileName;
}

void C_Route::setNumOfWayPoints(const int &newNumOfWayPoints)
{
	numOfWayPoints = newNumOfWayPoints;
}
	