// Created by Josh Heyde
// Last updated 27/01/09

#include "C_Wave.h"
#include "C_WorldEntity.h"
#include "C_Route.h"
#include <HAPI_lib.h>
#include <assert.h>

// constructor
C_Wave::C_Wave(const int &rRef, const int &id1, const int &id2, const int &id3, const int &id4, const int &id5)
: routeReference(rRef), enemyToSpawn(0), numOfEnemies(0)
{
	// storing the enemy IDs in the vector
	v_enemyIDVector.push_back(id1);
	numOfEnemies++;
	v_enemyIDVector.push_back(id2);
	numOfEnemies++;
	v_enemyIDVector.push_back(id3);
	numOfEnemies++;
	v_enemyIDVector.push_back(id4);
	numOfEnemies++;
	v_enemyIDVector.push_back(id5);
	numOfEnemies++;
}

// destructor
C_Wave::~C_Wave()
{
}

// mutator functions
void C_Wave::setEnemyToSpawn(const int &newEnemyToSpawn)
{
	enemyToSpawn = newEnemyToSpawn;
}

void C_Wave::setNumOfEnemies(const int &newNumOfEnemies)
{
	numOfEnemies = newNumOfEnemies;
}

void C_Wave::setRouteReference(const int &newRouteReference)
{
	routeReference = newRouteReference;
}

// sets up  every enemy
bool C_Wave::intialise(std::vector<C_WorldEntity*> v_entityVector, const C_Route* route)
{
	try
	{
		for(size_t i = 0; i < v_enemyIDVector.size(); i++)
		{
			// sets up the enemy
			v_entityVector[v_enemyIDVector[i]]->setCurrentPosition(route->getWayPoint(0));
			v_entityVector[v_enemyIDVector[i]]->setNextPosition(route->getWayPoint(0));
			v_entityVector[v_enemyIDVector[i]]->setCurrentWayPoint(1);
			v_entityVector[v_enemyIDVector[i]]->determineDirection(*route->getWayPoint(1));
			v_entityVector[v_enemyIDVector[i]]->resetHealth();
			v_entityVector[v_enemyIDVector[i]]->resetCentre();
		}
	}
	catch(char *error)
	{
		return false;
	}

	// signifies the wave has been successfully initialised
	return true;
}

// spawns an enemy from the wave
void C_Wave::spawnEnemy(std::vector <C_WorldEntity*> v_entityVector)
{
	// error handling statements
	// asserting certain statements for this function
	assert(v_entityVector.size() != false);
	assert(enemyToSpawn < numOfEnemies);

	v_entityVector[v_enemyIDVector[enemyToSpawn]]->setActive(true);
	enemyToSpawn++;
}

// checks to see if the wave has been completed
bool C_Wave::isWaveComplete(std::vector<C_WorldEntity*> v_entityVector) const
{
	// checks to see if all enemies are dead
	for(size_t i = 0; i < v_enemyIDVector.size(); i++)
	{
		if(v_entityVector[v_enemyIDVector[i]]->getActive())
			return false;
	}

	// signifies the wave has been completed
	return true;
}

// checks to see if all enemies have been spawned
const bool C_Wave::allEnemiesSpawned() const
{
	return(enemyToSpawn == numOfEnemies);
}

// allows a specific enemy to change it's current target to the next waypoint
void C_Wave::getNextWayPoint(std::vector<C_WorldEntity*> v_entityVector, const int &id, const C_Route* route)
{
	// increments the current waypoint
	v_entityVector[id]->setCurrentWayPoint(v_entityVector[id]->getCurrentWayPoint() + 1);

	// checks to see if the entity has reached the end of the route
	if(!(v_entityVector[id]->getCurrentWayPoint() >= route->getNumOfWayPoints()))
	{
		v_entityVector[id]->determineDirection(*route->getWayPoint(v_entityVector[id]->getCurrentWayPoint()));
	}
	else
	{
		// kills the enemy if it has completed it's route
		v_entityVector[id]->setActive(false);
	}
}

// updates the wave
void C_Wave::update(std::vector<C_WorldEntity*> v_entityVector, const C_Route* route)
{
	for(size_t i = 0; i < v_enemyIDVector.size(); i++)
	{
		// checks to see if the enemy has reached it's target
		if(v_entityVector[v_enemyIDVector[i]]->atTarget())
		{
			// if it has, it moves on to the next one
			getNextWayPoint(v_entityVector, v_enemyIDVector[i], route);
		}
	}
}

// resets the wave to be used again
void C_Wave::resetWave(std::vector<C_WorldEntity*> v_entityVector)
{
	enemyToSpawn = 0;		
}
